@@ -4,6 +4,7 @@ from meteor_reasoner.automata.satisfy import *
 from meteor_reasoner.materialization.coalesce import *
 from meteor_reasoner.automata.guess import *
 from meteor_reasoner.materialization.apply import *
+from collections import defaultdict
 
 
 class BuchiAutomata:
@@ -303,7 +304,7 @@ class BuchiAutomata:
            boolean
         """
         initial_window_len, initial_window_ruler_intervals = get_initial_ruler_intervals(self.points,
-            left_border=- self.x - self.z, right_border= -self.x + self.z, gcd=self.gcd)
+            left_border=-self.x - self.z, right_border = -self.x + self.z, gcd=self.gcd)
 
         for l_min, r_min in self.search_window(initial_window_ruler_intervals):
             left_flag = self.search(l_min, "left", self.left_conditions)
@@ -459,13 +460,12 @@ class BuchiAutomata:
                 else:
                     raise Exception("Please convert Diamondminus or Diamondplus to Boxminus or Boxplus")
 
-
         if len(binary_literals) != 0:
             guess_points, left_right_literals = generate_guess_points(w, binary_literals, ruler_intervals, self.z, self.gcd)
             # guess existing intervals
             for literal in left_right_literals:
                 if literal.get_predicate() in self.D and literal.get_entity() is None:
-                    guess_D[literal.get_predicate()] = D[literal.get_predicate()]
+                    guess_D[literal.get_predicate()] = self.D[literal.get_predicate()]
                 elif literal.get_predicate() in self.D and literal.get_entity() in self.D[literal.get_predicate()]:
                     guess_D[literal.get_predicate()][literal.get_entity()] = self.D[literal.get_predicate()][literal.get_entity()]
                 else:
