@@ -1,9 +1,8 @@
 from meteor_reasoner.classes.literal import *
 from functools import reduce
-from collections import defaultdict
-from meteor_reasoner.classes.term import *
 import math
 from meteor_reasoner.classes.interval import Interval
+from meteor_reasoner.classes.atom import Atom
 
 
 def construct_left_right_pattern(points, gcd):
